@@ -7,7 +7,7 @@
 -- ----------------------------------------------------------------------------}
 
 module ParseMonad (
-  	AlexInput, alexInputPrevChar, alexGetChar,
+  	AlexInput, alexInputPrevChar, alexGetChar, alexGetByte,
   	AlexPosn(..), alexStartPos,
  
 	P, runP, StartCode, failP, lookupSMac, lookupRMac, newSMac, newRMac,
@@ -18,21 +18,34 @@ import AbsSyn hiding ( StartCode )
 import CharSet ( CharSet )
 import Map ( Map )
 import qualified Map hiding ( Map )
-
+import UTF8
+import Data.Word (Word8)
 -- -----------------------------------------------------------------------------
 -- The input type
+--import Codec.Binary.UTF8.Light as UTF8
+
+type Byte = Word8
 
 type AlexInput = (AlexPosn, 	-- current position,
 		  Char,		-- previous char
+                  [Byte],
 		  String)	-- current input string
 
 alexInputPrevChar :: AlexInput -> Char
-alexInputPrevChar (_,c,_) = c
+alexInputPrevChar (_,c,_,_) = c
+
 
 alexGetChar :: AlexInput -> Maybe (Char,AlexInput)
-alexGetChar (_,_,[]) = Nothing
-alexGetChar (p,_,(c:s))  = let p' = alexMove p c in p' `seq`
-				Just (c, (p', c, s))
+alexGetChar (_,_,[],[]) = Nothing
+alexGetChar (p,_,[],(c:s))  = let p' = alexMove p c in p' `seq`
+				Just (c, (p', c, [], s))
+
+alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
+alexGetByte (p,c,(b:bs),s) = Just (b,(p,c,bs,s))
+alexGetByte (_,_,[],[]) = Nothing
+alexGetByte (p,_,[],(c:s))  = let p' = alexMove p c 
+                                  (b:bs) = UTF8.encode c
+                              in p' `seq`  Just (b, (p', c, bs, s))
 
 -- -----------------------------------------------------------------------------
 -- Token positions
@@ -84,10 +97,10 @@ runP str (senv,renv) (P p)
 	Right (_,a) -> Right a
  where initial_state = 
  	  PState{ smac_env=senv, rmac_env=renv,
-	     startcode = 0, input=(alexStartPos,'\n',str) }
+	     startcode = 0, input=(alexStartPos,'\n',[],str) }
 
 failP :: String -> P a
-failP str = P $ \PState{ input = (p,_,_) } -> Left (Just p,str)
+failP str = P $ \PState{ input = (p,_,_,_) } -> Left (Just p,str)
 
 -- Macros are expanded during parsing, to simplify the abstract
 -- syntax.  The parsing monad passes around two environments mapping
