@@ -4,33 +4,69 @@
 -- This code is in the PUBLIC DOMAIN; you may copy it freely and use
 -- it for any purpose whatsoever.
 
+import Data.Word (Word8)
 #if defined(ALEX_BASIC_BYTESTRING) || defined(ALEX_POSN_BYTESTRING) || defined(ALEX_MONAD_BYTESTRING)
 
-import qualified Data.ByteString.Lazy.Char8 as ByteString
+import qualified Data.Char
+import qualified Data.ByteString.Lazy     as ByteString
+import qualified Data.ByteString.Internal as ByteString (w2c)
 
 #elif defined(ALEX_STRICT_BYTESTRING)
 
-import qualified Data.ByteString.Char8    as ByteString
+import qualified Data.Char
+import qualified Data.ByteString          as ByteString
 import qualified Data.ByteString.Internal as ByteString
 import qualified Data.ByteString.Unsafe   as ByteString
 
+#else
+
+import qualified Data.Bits
+
+-- | Encode a Haskell String to a list of Word8 values, in UTF8 format.
+utf8Encode :: Char -> [Word8]
+utf8Encode = map fromIntegral . go . ord
+ where
+  go oc
+   | oc <= 0x7f       = [oc]
+
+   | oc <= 0x7ff      = [ 0xc0 + (oc `Data.Bits.shiftR` 6)
+                        , 0x80 + oc Data.Bits..&. 0x3f
+                        ]
+
+   | oc <= 0xffff     = [ 0xe0 + (oc `Data.Bits.shiftR` 12)
+                        , 0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
+                        , 0x80 + oc Data.Bits..&. 0x3f
+                        ]
+   | otherwise        = [ 0xf0 + (oc `Data.Bits.shiftR` 18)
+                        , 0x80 + ((oc `Data.Bits.shiftR` 12) Data.Bits..&. 0x3f)
+                        , 0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
+                        , 0x80 + oc Data.Bits..&. 0x3f
+                        ]
+
 #endif
 
+type Byte = Word8
+
 -- -----------------------------------------------------------------------------
 -- The input type
 
 #if defined(ALEX_POSN) || defined(ALEX_MONAD) || defined(ALEX_GSCAN)
 type AlexInput = (AlexPosn,     -- current position,
                   Char,         -- previous char
+                  [Byte],       -- pending bytes on current char
                   String)       -- current input string
 
-alexInputPrevChar :: AlexInput -> Char
-alexInputPrevChar (p,c,s) = c
+ignorePendingBytes (p,c,ps,s) = (p,c,s)
 
-alexGetChar :: AlexInput -> Maybe (Char,AlexInput)
-alexGetChar (p,c,[]) = Nothing
-alexGetChar (p,_,(c:s))  = let p' = alexMove p c in p' `seq`
-                                Just (c, (p', c, s))
+alexInputPrevChar :: AlexInput -> Char
+alexInputPrevChar (p,c,bs,s) = c
+
+alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
+alexGetByte (p,c,(b:bs),s) = Just (b,(p,c,bs,s))
+alexGetByte (p,c,[],[]) = Nothing
+alexGetByte (p,_,[],(c:s))  = let p' = alexMove p c 
+                                  (b:bs) = utf8Encode c
+                              in p' `seq`  Just (b, (p', c, bs, s))
 #endif
 
 #if defined(ALEX_POSN_BYTESTRING) || defined(ALEX_MONAD_BYTESTRING)
@@ -41,12 +77,42 @@ type AlexInput = (AlexPosn,     -- current position,
 alexInputPrevChar :: AlexInput -> Char
 alexInputPrevChar (p,c,s) = c
 
-alexGetChar :: AlexInput -> Maybe (Char,AlexInput)
-alexGetChar (p,_,cs) | ByteString.null cs = Nothing
-                     | otherwise = let c   = ByteString.head cs
+alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
+alexGetByte (p,_,cs) | ByteString.null cs = Nothing
+                     | otherwise = let b   = ByteString.head cs
                                        cs' = ByteString.tail cs
+                                       c   = ByteString.w2c b
                                        p'  = alexMove p c
-                                    in p' `seq` cs' `seq` Just (c, (p', c, cs'))
+                                    in p' `seq` cs' `seq` Just (b, (p', c, cs'))
+#endif
+
+#ifdef ALEX_BASIC_BYTESTRING
+type AlexInput = (Char,
+                  ByteString.ByteString)
+
+alexInputPrevChar :: AlexInput -> Char
+alexInputPrevChar (c,_) = c
+
+alexGetByte (_, cs)
+   | ByteString.null cs = Nothing
+   | otherwise          = Just (ByteString.head cs,
+                                (ByteString.w2c $ ByteString.head cs,
+                                 ByteString.tail cs))
+#endif
+
+#ifdef ALEX_STRICT_BYTESTRING
+data AlexInput = AlexInput { alexChar :: {-# UNPACK #-}!Char
+                           , alexStr  :: {-# UNPACK #-}!ByteString.ByteString }
+
+alexInputPrevChar :: AlexInput -> Char
+alexInputPrevChar = alexChar
+
+alexGetByte (AlexInput _ cs)
+    | ByteString.null cs = Nothing
+    | otherwise          = Just $!  (ByteString.head cs, AlexInput c cs')
+    where
+        (c,cs') = (ByteString.w2c (ByteString.unsafeHead cs)
+                  , ByteString.unsafeTail cs)
 #endif
 
 -- -----------------------------------------------------------------------------
@@ -80,6 +146,7 @@ data AlexState = AlexState {
         alex_pos :: !AlexPosn,  -- position at current input location
         alex_inp :: String,     -- the current input
         alex_chr :: !Char,      -- the character before the input
+        alex_bytes :: [Byte],
         alex_scd :: !Int        -- the current startcode
 #ifdef ALEX_MONAD_USER_STATE
       , alex_ust :: AlexUserState -- AlexUserState will be defined in the user program
@@ -93,6 +160,7 @@ runAlex input (Alex f)
    = case f (AlexState {alex_pos = alexStartPos,
                         alex_inp = input,       
                         alex_chr = '\n',
+                        alex_bytes = [],
 #ifdef ALEX_MONAD_USER_STATE
                         alex_ust = alexInitUserState,
 #endif
@@ -109,12 +177,12 @@ instance Monad Alex where
 
 alexGetInput :: Alex AlexInput
 alexGetInput
- = Alex $ \s@AlexState{alex_pos=pos,alex_chr=c,alex_inp=inp} -> 
-        Right (s, (pos,c,inp))
+ = Alex $ \s@AlexState{alex_pos=pos,alex_chr=c,alex_bytes=bs,alex_inp=inp} -> 
+        Right (s, (pos,c,bs,inp))
 
 alexSetInput :: AlexInput -> Alex ()
-alexSetInput (pos,c,inp)
- = Alex $ \s -> case s{alex_pos=pos,alex_chr=c,alex_inp=inp} of
+alexSetInput (pos,c,bs,inp)
+ = Alex $ \s -> case s{alex_pos=pos,alex_chr=c,alex_bytes=bs,alex_inp=inp} of
                   s@(AlexState{}) -> Right (s, ())
 
 alexError :: String -> Alex a
@@ -137,12 +205,12 @@ alexMonadScan = do
         alexMonadScan
     AlexToken inp' len action -> do
         alexSetInput inp'
-        action inp len
+        action (ignorePendingBytes inp) len
 
 -- -----------------------------------------------------------------------------
 -- Useful token actions
 
-type AlexAction result = AlexInput -> Int -> result
+type AlexAction result = AlexInput -> Int -> Alex result
 
 -- just ignore this token and scan another one
 -- skip :: AlexAction result
@@ -153,10 +221,10 @@ skip input len = alexMonadScan
 begin code input len = do alexSetStartCode code; alexMonadScan
 
 -- perform an action for this token, and set the start code to a new value
--- andBegin :: AlexAction result -> Int -> AlexAction result
+andBegin :: AlexAction result -> Int -> AlexAction result
 (action `andBegin` code) input len = do alexSetStartCode code; action input len
 
--- token :: (String -> Int -> token) -> AlexAction token
+token :: (AlexInput -> Int -> token) -> AlexAction token
 token t input len = return (t input len)
 #endif /* ALEX_MONAD */
 
@@ -226,12 +294,12 @@ alexMonadScan = do
         alexMonadScan
     AlexToken inp' len action -> do
         alexSetInput inp'
-        action inp len
+        action (ignorePendingBytes inp) len
 
 -- -----------------------------------------------------------------------------
 -- Useful token actions
 
-type AlexAction result = AlexInput -> Int -> result
+type AlexAction result = AlexInput -> Int -> Alex result
 
 -- just ignore this token and scan another one
 -- skip :: AlexAction result
@@ -242,10 +310,10 @@ skip input len = alexMonadScan
 begin code input len = do alexSetStartCode code; alexMonadScan
 
 -- perform an action for this token, and set the start code to a new value
--- andBegin :: AlexAction result -> Int -> AlexAction result
+andBegin :: AlexAction result -> Int -> AlexAction result
 (action `andBegin` code) input len = do alexSetStartCode code; action input len
 
--- token :: (String -> Int -> token) -> AlexAction token
+token :: (AlexInput -> Int -> token) -> AlexAction token
 token t input len = return (t input len)
 #endif /* ALEX_MONAD_BYTESTRING */
 
@@ -254,21 +322,25 @@ token t input len = return (t input len)
 -- Basic wrapper
 
 #ifdef ALEX_BASIC
-type AlexInput = (Char,String)
+type AlexInput = (Char,[Byte],String)
 
-alexGetChar (_, [])   = Nothing
-alexGetChar (_, c:cs) = Just (c, (c,cs))
 
 alexInputPrevChar (c,_) = c
 
 -- alexScanTokens :: String -> [token]
-alexScanTokens str = go ('\n',str)
-  where go inp@(_,str) =
+alexScanTokens str = go ('\n',[],str)
+  where go inp@(_,_bs,str) =
           case alexScan inp 0 of
                 AlexEOF -> []
                 AlexError _ -> error "lexical error"
                 AlexSkip  inp' len     -> go inp'
                 AlexToken inp' len act -> act (take len str) : go inp'
+
+alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
+alexGetByte (c,(b:bs),s) = Just (b,(c,bs,s))
+alexGetByte (c,[],[])    = Nothing
+alexGetByte (_,[],(c:s)) = case utf8Encode c of
+                             (b:bs) -> Just (b, (c, bs, s))
 #endif
 
 
@@ -276,12 +348,6 @@ alexScanTokens str = go ('\n',str)
 -- Basic wrapper, ByteString version
 
 #ifdef ALEX_BASIC_BYTESTRING
-type AlexInput = (Char,ByteString.ByteString)
-
-alexGetChar (_, cs) | ByteString.null cs = Nothing
-                    | otherwise          = Just (ByteString.head cs, (ByteString.head cs, ByteString.tail cs))
-
-alexInputPrevChar (c,_) = c
 
 -- alexScanTokens :: String -> [token]
 alexScanTokens str = go ('\n',str)
@@ -297,18 +363,6 @@ alexScanTokens str = go ('\n',str)
 
 #ifdef ALEX_STRICT_BYTESTRING
 
-data AlexInput = AlexInput { alexChar :: {-# UNPACK #-}!Char
-                           , alexStr  :: {-# UNPACK #-}!ByteString.ByteString }
-
-alexGetChar (AlexInput _ cs)
-    | ByteString.null cs = Nothing
-    | otherwise          = Just $!  (ByteString.head cs, AlexInput c cs')
-    where
-        (c,cs') = (ByteString.w2c (ByteString.unsafeHead cs)
-                  , ByteString.unsafeTail cs)
-
-alexInputPrevChar = alexChar
-
 -- alexScanTokens :: String -> [token]
 alexScanTokens str = go (AlexInput '\n' str)
   where go inp@(AlexInput _ str) =
@@ -328,11 +382,11 @@ alexScanTokens str = go (AlexInput '\n' str)
 
 #ifdef ALEX_POSN
 --alexScanTokens :: String -> [token]
-alexScanTokens str = go (alexStartPos,'\n',str)
-  where go inp@(pos,_,str) =
+alexScanTokens str = go (alexStartPos,'\n',[],str)
+  where go inp@(pos,_,_,str) =
           case alexScan inp 0 of
                 AlexEOF -> []
-                AlexError ((AlexPn _ line column),_,_) -> error $ "lexical error at " ++ (show line) ++ " line, " ++ (show column) ++ " column"
+                AlexError ((AlexPn _ line column),_,_,_) -> error $ "lexical error at " ++ (show line) ++ " line, " ++ (show column) ++ " column"
                 AlexSkip  inp' len     -> go inp'
                 AlexToken inp' len act -> act pos (take len str) : go inp'
 #endif
@@ -359,14 +413,14 @@ alexScanTokens str = go (alexStartPos,'\n',str)
 -- For compatibility with previous versions of Alex, and because we can.
 
 #ifdef ALEX_GSCAN
-alexGScan stop state inp = alex_gscan stop alexStartPos '\n' inp (0,state)
+alexGScan stop state inp = alex_gscan stop alexStartPos '\n' [] inp (0,state)
 
-alex_gscan stop p c inp (sc,state) =
-  case alexScan (p,c,inp) sc of
+alex_gscan stop p c bs inp (sc,state) =
+  case alexScan (p,c,bs,inp) sc of
         AlexEOF     -> stop p c inp (sc,state)
         AlexError _ -> stop p c inp (sc,state)
-        AlexSkip (p',c',inp') len -> alex_gscan stop p' c' inp' (sc,state)
-        AlexToken (p',c',inp') len k ->
-             k p c inp len (\scs -> alex_gscan stop p' c' inp' scs)
+        AlexSkip (p',c',bs',inp') len -> alex_gscan stop p' c' bs' inp' (sc,state)
+        AlexToken (p',c',bs',inp') len k ->
+             k p c inp len (\scs -> alex_gscan stop p' c' bs' inp' scs)
                 (sc,state)
 #endif
